!vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv!
!                                                                      !
!  Module name: USR_RATES                                              !
!                                                                      !
!  Purpose: Hook for user defined reaction rates.                      !
!                                                                      !
!  Author: J.Musser                                   Date: 10-Oct-12  !
!                                                                      !
!  Comments: Write reaction rates in units of moles/sec.cm^3 (cgs) or  !
!  kmoles/sec.m^3 (SI). Units should match those specified in the data !
!  file.
!                                                                      !
!  Example reaction: Methane combustion                                !
!                                                                      !
!  mfix.dat input:                                                     !
!``````````````````````````````````````````````````````````````````````!
!    @(RXNS)                                                           !
!      CH4_Comb { chem_eq = "CH4 + 2.0*O2 --> CO2 + 2.0*H2O" }         !
!    @(END)                                                            !
!``````````````````````````````````````````````````````````````````````!
!                                                                      !
!  usr_rates.f input:                                                  !
!``````````````````````````````````````````````````````````````````````!
!    c_O2  = (RO_g(IJK)*X_g(IJK,O2)/MW_g(O2))                          !
!    c_CH4 = (RO_g(IJK)*X_g(IJK,CH4)/MW_g(CH4))                        !
!    RATES(CH4_Comb) = 2.0d5 * EP_g(IJK) * c_O2 * c_CH4                !
!``````````````````````````````````````````````````````````````````````!
!  * Species alias and reaction names given in the data file can be    !
!    used in reference to the reaction index in RATES and a species    !
!    index in gas/solids phase variables.                              !
!                                                                      !
!  * Additional information is provided in section 5.11 of the code    !
!    Readme.                                                           !
!                                                                      !
!^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^!
      SUBROUTINE USR_RATES(IJK, RATES)

      USE compar
      USE constant
      USE energy
      USE fldvar
      USE fun_avg
      USE functions
      USE funits
      USE geometry
      USE indices
      USE parallel
      USE param
      USE param1
      USE physprop
      USE run
      USE rxns
      USE sendrecv
      USE toleranc
      USE usr


      IMPLICIT NONE

      INTEGER, INTENT(IN) :: IJK

      DOUBLE PRECISION, DIMENSION(NO_OF_RXNS), INTENT(OUT) :: RATES(NO_OF_RXNS)


   INCLUDE 'species.inc'

! Reaction specific variables:
!`````````````````````````````````````````````````````````````````````//
      DOUBLE PRECISION Tg   ! gas phase
    
! Gas phase concentrations (mol/m^3)
      DOUBLE PRECISION c_CH4, c_H2O, c_CO, c_H2, c_CO2
! Partial pressures (KPa)
      DOUBLE PRECISION p_CH4, p_H2O, p_CO, p_H2, p_CO2, Pg_KPa

! Temporary values for reversible reactions
   DOUBLE PRECISION FWD, RVS, NET
! Prevent reactions when mass fraction falls below c_Limiter.
     DOUBLE PRECISION, parameter :: c_Limiter = 1.0d-7

Tg=MIN(TMAX, T_g(IJK))
! Convert gas pressure to KPa
!      Pg_KPa = 1.0d-3 * P_g(IJK)  !  KPa

! Initialize partial pressures and concentrations.
      c_CH4  = ZERO;   p_CH4 = ZERO;
      c_H2O  = ZERO;   p_H2O = ZERO;
      c_CO    = ZERO;   p_CO   = ZERO;
      c_H2 = ZERO;       p_H2 = zero;
      c_CO2 = ZERO;     p_CO2 = zero;

! Calculate the partial pressure and molar concentration of CH4
      IF(X_g(IJK,CH4) > c_Limiter) then
         p_CH4 = P_g(IJK) * X_g(IJK,CH4) * (MW_MIX_g(IJK) / MW_g(CH4))
         c_CH4 = RO_g(IJK) * X_g(IJK,CH4) / Mw_g(CH4)
      ENDIF

! Calculate the partial pressure and molar concentration of H2O
      IF(X_g(IJK,H2O) > c_Limiter) then
         p_H2O = P_g(IJK) * X_g(IJK,H2O) * (MW_MIX_g(IJK) / MW_g(H2O))
         c_H2O = RO_g(IJK)*X_g(IJK,H2O)/Mw_g(H2O)
      ENDIF
! Calculate the partial pressure and molar concentration of CO
      IF(X_g(IJK,CO) > c_Limiter) then
         p_CO = P_g(IJK) * X_g(IJK,CO) * (MW_MIX_g(IJK) / MW_g(CO))
         c_CO = RO_g(IJK)*X_g(IJK,CO)/Mw_g(CO)
      ENDIF
! Calculate the partial pressure and molar concentration of CO2
      IF(X_g(IJK,CO2) > c_Limiter) then
         p_CO2 = P_g(IJK) * X_g(IJK,CO2) * (MW_MIX_g(IJK) / MW_g(CO2))
         c_CO2 = RO_g(IJK)*X_g(IJK,CO2)/Mw_g(CO2)
      ENDIF
! Calculate the partial pressure and molar concentration of H2
      IF(X_g(IJK,H2) > c_Limiter) then
         p_H2 = P_g(IJK) * X_g(IJK,H2) * (MW_MIX_g(IJK) / MW_g(H2))
         c_H2 = RO_g(IJK)*X_g(IJK,H2)/Mw_g(H2)
      ENDIF

! Reaction rates:
!`````````````````````````````````````````````````````````````````````//
! Include reaction rates here. Reaction rates should be stored in the
! variable RATES. The reaction name given in the data file can be used
! to store the rate in the appropriate array location. Additional
! input format parameters are given in Section 4.11 of the code Readme.
!                         Homogeneous Reactions                        !


!---------------------------------------------------------------------//
! R3 : CH4 + H2O --> CO + 3H2   (kmole/m^3.s)
!if ((X_g(IJK,CH4) /= c_Limiter) .and. (X_g(IJK,H2O) /= c_Limiter)) THEN
RATES(R3)= 3.0d9*c_CH4*c_H2O*exp(-15154.6d0/Tg) ! (kmole/m^3.s) 
!else
!RATES(R3)=0.0
!ENDIF

! R4F CO + H2O--> CO2 + H2  (kmole/m^3.s)

!if ((X_g(IJK,CO) /= c_Limiter) .and. (X_g(IJK,H2O) /= c_Limiter)) THEN
RATES(R4F)=2.8d3*c_CO*c_H2O*exp(-1515.5d0/Tg) ! (kmole/m^3.s)
!else
!FWD=0.0
!ENDIF

! R4R CO + H2O<-- CO2 + H2  (kmole/m^3.s)
!if ((X_g(IJK,CO2) /= c_Limiter) .and. (X_g(IJK,H2) /= c_Limiter)) THEN
!RVS=9.6d4*c_CO2*c_H2*exp(-5604.8d0/Tg)  ! (kmole/m^3.s)
!else
!RVS=0.0
!ENDIF

 !NET = FWD - RVS
  !    IF(NET >= ZERO) THEN
   !      RATES(R4F) =  NET
    !     RATES(R4R) =  ZERO
     ! ELSE
      !   RATES(R4F) =  ZERO
       !  RATES(R4R) = -NET
      !ENDIF

! Save reaction rates for visualization
      IF(nrr>=R3) ReactionRates(IJK,R3) = RATES(R3)
      IF(nrr>=R4F) ReactionRates(IJK,R4F) = RATES(R4F)
   !   IF(nrr>=R4R) ReactionRates(IJK,R4R) = RATES(R4R)
     
      RETURN

      END SUBROUTINE USR_RATES
