!vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv!
!                                                                      !
!  Module name: USR_RATES                                              !
!                                                                      !
!  Purpose: Hook for user defined reaction rates.                      !
!                                                                      !
!  Author: J.Musser                                   Date: 10-Oct-12  !
!                                                                      !
!  Comments: Write reaction rates in amount of chemical substance      !
!  per time. For CGS this has units mol/s. For _MFIX_ SI this has      !
!  units kmol/s.                                                       !
!                                                                      !
!  WARNING: Only discrete phase reactions should be specified here.     !
!  Homogeneous gas phase reactions in DEM simulations must be given    !
!  in the continuum reaction hook (usr_rates.f).                       !
!                                                                      !
!  The call to usr_rates_des is made from inside a particle loop which !
!  is nested inside an IJK loop. Fluid grid calculations independent   !
!  of particle properties can be carried out in des/usr4_des.f to      !
!  reduce redundant calculations.                                      !
!                                                                      !
!  Example: Evaporation                                                !
!                                                                      !
!  mfix.dat input:                                                     !
!``````````````````````````````````````````````````````````````````````!
!    @(DES_RXNS)                                                       !
!      Evap { chem_eq = "Liquid --> Vapor" }                           !
!    @(DES_END)                                                        !
!``````````````````````````````````````````````````````````````````````!
!                                                                      !
!  des/usr_rates_des.f input:                                          !
!``````````````````````````````````````````````````````````````````````!
!    Sa = Pi*(2.0d0*DES_RADIUS(NP))**2    ! Particle surface area      !
!    H2O_xfr = ...  ! An expression for mass transfer coefficient      !
!    Cmg_H2O = ...  ! Molar concentration grad of water vapor          !
!    DES_RATES(EVAP) = Sa * H2O_xfr * Cmg_H2O                          !
!``````````````````````````````````````````````````````````````````````!
!  * Species alias and reaction names given in the data file can be    !
!    used in reference to the reaction index in DES_RATES and a        !
!    species index in gas/solids phase variables.                      !
!                                                                      !
!  * Additional information is provided in section 4.11 of the code    !
!    Readme.                                                           !
!                                                                      !
!^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^!
      SUBROUTINE USR_RATES_DES(NP, pM, IJK, DES_RATES)

      USE compar
      USE constant
      USE des_thermo
      USE des_rxns
      USE discretelement
      USE energy
      USE fldvar
      USE funits
      USE geometry
      USE indices
      USE param
      USE param1
      USE physprop
      USE rxns
      USE run
      USE usr
      USE fun_avg
      USE functions


      IMPLICIT NONE

      INTEGER, INTENT(IN) :: NP  ! Global index of particle
      INTEGER, INTENT(IN) :: pM  ! Solid phase index of particle NP
      INTEGER, INTENT(IN) :: IJK ! Fluid cell index containing NP

! Calculated reaction rates. (reacted moles per sec)
      DOUBLE PRECISION, INTENT(OUT) :: DES_RATES(NO_OF_DES_RXNS)


! Reaction specific variables:
!`````````````````````````````````````````````````````````````````````//

!     Uncomment the following line when customizing this usr_rates_des.f to include generated file species.inc:
!
     INCLUDE 'species.inc'
!Variable declaration
DOUBLE PRECISION Ap ! particle surface area
DOUBLE PRECISION Dp ! particle dia
DOUBLE PRECISION Vp ! particle vol
DOUBLE PRECISION Tp, Tg, TMAX !particle temp , gas temp
DOUBLE PRECISION pH2O, pCO2 !
DOUBLE PRECISION rkin1, rdiff1, rkin2, rdiff2
DOUBLE PRECISION, parameter :: c_Limiter = 1.0d-7

INTEGER :: lB, uB

! Only do reaction on phase 2 (biomass)
if(pM==1) Return


Tp=DES_T_s(NP)
Tg=MIN(TMAX, T_g(IJK))
Dp=2.0*DES_RADIUS(NP)
Ap=Pi*Dp*Dp !sphere surface area
Vp=4.0/3.0*Pi*(Dp/2.0)**3.0



!!!!!!!!!!!!!!!!!!!! drying !!!!!!!!!!!!!!!!!!!
!if (DES_X_s(NP,H2O_L) /= c_Limiter) then
DES_RATES(drying)= 5.1d6*exp(-10572.2d0/Tp)*(DES_X_s(NP,H2O_L)*ro_s0(2)*Vp)/(MW_s(pM,H2O_L)) ! 
!else
!DES_RATES(drying)=0.0
!ENDIF


!!!!!!!!!!!!!!!!!!!!!! pyrolysis dry biomass (wood) !!!!!!!!!!!!!!!!!!!
!if (DES_X_s(NP,wood) /= c_Limiter) then
DES_RATES(pyr)= 5.0d6*exp(-14433.0d0/Tp)*(DES_X_s(NP,wood)*ro_s0(2)*Vp)/(MW_s(pM,wood))
!else
!DES_RATES(pyr)=0.0
!ENDIF



!!!!!!!!!!!!!!!!!!!!!! R1 C+CO2 --> 2CO !!!!!!!!!!!!!!!!!!!!!!
!if ((DES_X_s(NP,Char) /= c_Limiter) .and. (X_g(IJK,CO2) /= c_Limiter)) then
rkin1=8.3d0*Tp*exp(-5256.0d0/Tp) ! (s/m)
rdiff1= 5.0d-12*(((Tp+Tg)/2.0)**0.75)/(Dp) ! (s/m)
pCO2= (P_g(IJK)*MW_MIX_g(IJK)*X_g(IJK,CO2)/MW_g(CO2)) ! (pa = kg/ms2)
DES_RATES(R1)=Ap*pCO2*((rdiff1*rkin1)/(rdiff1+rkin1))/(MW_s(pM,Char)) ! (kmole/s)
!else
!DES_RATES(R1)=0.0
!ENDIF

!!!!!!!!!!!!!!!!!!!!!! R2 C+H2O --> CO + H2!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!if ((DES_X_s(NP,Char) /= c_Limiter) .and. (X_g(IJK,H2O) /= c_Limiter)) then
rkin2=45.6d0*Tp*exp(-5256.0d0/Tp) ! (s/m)
rdiff2= 5.0d-12*(((Tp+Tg)/2.0)**0.75)/(Dp)
pH2O= (P_g(IJK)*MW_MIX_g(IJK)*X_g(IJK,H2O)/MW_g(H2O)) ! (pa = kg/ms2)
DES_RATES(R2)=Ap*pH2O*((rdiff2*rkin2)/(rdiff2+rkin2))/(MW_s(pM,Char)) ! (kmole/s) 
!else
!DES_RATES(R2)=0.0
!ENDIF

! devol
!if (DES_X_s(NP,VM) /= 0) then
!DES_RATES(devol)= 5.0d6*exp(-14433d0/Tp)*(DES_X_s(NP,VM)*ro_s0(2)*Vp*0.88d0)/(MW_s(pM,VM)) ! 
!else
!DES_RATES(devol)=0
!ENDIF


!**********************************************************************!
! Save reaction rates for visualization
!**********************************************************************!
! Option 1: Save data at the cell level
!           Need to acumulate contribution of each particle in cell IJK
!           ReactionRates needs to be reset before the particle loop in
!           rxns_gs_des1.f 
!           The first 3 slots of ReactionRates store the homogeneous reaction
!           (NO_OF_RXNS) rates, the next 4 slots store the heterogeneous 
!           reaction (NO_OF_DES_RXNS) rates


      lb = NO_OF_RXNS +1
      uB = NO_OF_RXNS + NO_OF_DES_RXNS
      ReactionRates(IJK,lB:uB) = ReactionRates(IJK,lB:uB) + DES_RATES(:)

! Option 2: Save data at the particle level
!           Use des-usr_var array

      des_usr_var(1,NP) = DES_RATES(R1) 
      des_usr_var(2,NP) = DES_RATES(R2) 
      des_usr_var(3,NP) = DES_RATES(drying) 
      des_usr_var(4,NP) = DES_RATES(pyr) 


      RETURN

      END SUBROUTINE USR_RATES_DES
